      SUBROUTINE trans(aer_dtau, aer_gg, aer_ssa, 
     $     albedo, dens, 
     $     ice_dtau, ice_gg, ice_ssa, lambda_t,
     $     ndens, nlambda_t, nlyr, nzout, maxphi, maxumu,
     $     nstr, numu, nphi, path, npath, press, solver, 
     $     deltam, o3_type, sza, phi0, temper, umu,
     $     nprndis, prntdis,
     $     wc_dtau, wc_gg, wc_ssa, zd, zout, phi, utau,
     $     dfdt, flup, rfldir, rfldn,  uavg, uavgdn, uavgso, uavgup,
     $     u0u, albmed, trnmed, uu )
*
      IMPLICIT LOGICAL (A-Z)    ! To nearly get strong typing
      INCLUDE 'param.inc'
*
*BOD
*DESCRIPTION
* Calculates photodissociation rates.
*USAGE
**FORTRAN
*      CALL phodis(dens, ndens, nlyr, press,
*     $     temper, zd )  
**PERL
*INPUT
**dens
*     real array(levels,species)
*     The density of ndens species at nlyr+1 levels. The species index
*     must corresponds to the following trace gases if available. Water
*     vapor, carbon dioxide and nitrogen dioxide are presently not in
*     use and may be set to zero.
***1   Air
***2   ozone (o3)
***3   oxygen (o2)
***4   water vapor (h2o)
***5   carbon dioxide (co2)
***6   nitrogen dioxide (no2)
**ndens
*     integer
*     Number of densities in the model atmosphere. See dens.
**nlyr
*     integer
*     Number of layers in the model atmosphere.
**press
*     real array(0:*)
*     The pressure in mb at levels. 
**temper
*     real array(*)
*     The temperatures (levels in the atmosphere) at which
*     the ozone cross section is to be calculated.
**zd
*     real array(0:*)
*     The altitude in km for levels. 
*OUTPUT
*ERROR_MESSAGES
*ROUTINES_CALLED
*     crs_o3
*     rayleigh
*FILES
*     crs_o3_Molina.dat        Data file
*     consts.inc               Include file
*     param.inc                Include file
*REFERENCES
*EOD
*
      CHARACTER*(*) path     ! No. of characters must equal 
                             ! length of pathname            
      INTEGER ndens, nlambda_t, nlyr, npath, nprndis, nzout,
     $     o3_type, prntdis(*), solver
      REAL*4 aer_dtau(nlyr,*), aer_gg(nlyr,*), 
     $     aer_ssa(nlyr,*), albedo(*), dens(0:nlyr,*), 
     $     ice_dtau(nlyr,*), ice_gg(nlyr,*), 
     $     ice_ssa(nlyr,*), lambda_t(*), 
     $     press(0:*), temper(0:*),
     $     sza(*), wc_dtau(nlyr,*), wc_gg(nlyr,*), 
     $     wc_ssa(nlyr,*), zd(0:*), zout(*)
*
* Internal variables
*
      INTEGER  error, lc, lu, ip, iu, iv, intgr, type
      LOGICAL first, first_crs
      REAL abs_crs(0:maxcly, maxpro, maxwvn), 
     $     pi, ray_crs(maxwvn)
*
* Radiative transfer equation (RTE) solver parameters.
*
      CHARACTER header*127
      INTEGER ibcnd, ierror_d(47), ierror_s(33), ierror_t(22), imthd,
     $     maxphi, maxumu, ncoef, nil, nphi, nstr, ntau, numu
      LOGICAL deltam, newgeo, lamber, onlyfl, planck, 
     $     prndis(7), prntwo(2), quiet, spher, usrtau, usrang
      REAL accur, beta(0:maxcly), btemp, dtauc(maxcly), 
     $     fbeam, fisot, gg(maxcly), HL( 0:MAXCMU ), phi(*), phi0(*),
     $     pmom(0:maxcmu,maxcly), radius, ssalb(maxcly), temis, 
     $     ttemp, umu(*), umu0, utau(*), wvnmhi, 
     $     wvnmlo
      REAL dfdt(nzout, *), flup(nzout, *),
     $     rfldir(nzout, *), rfldn(nzout, *), 
     $     uavg(nzout, *), uavgdn(nzout, *),
     $     uavgso(nzout, *), uavgup(nzout, *),
     $     albmed( * ), trnmed( * )
*     $     ALBMED( MAXUMU ), TRNMED( MAXUMU )
      REAL u0u(maxumu, nzout, *), uu( maxumu, maxphi, nzout, * ) 
*
      DATA first /.TRUE./, first_crs /.TRUE./
      SAVE first, first_crs, pi
*
      IF ( first ) THEN
         pi = 2.0 * ASIN(1.0)
      ENDIF
*
*     SET VARIOUS INPUT DATA
*
      intgr       = 0
*
* Set RTE (Radiative Transfer Equation) solver input, technical stuff 
*
      accur       = 1.0E-3
      btemp       = 0.0
*      deltam      = .TRUE.
*      deltam      = .FALSE.
      fbeam       = 1.0  ! Calculate only transmittance first.
      fisot       = 0.0
      header      = ' '
      ibcnd       = 0
      lamber      = .TRUE.
      newgeo      = .TRUE.
      nil         = 0
      ncoef       = nstr
      imthd       = 0
      onlyfl      = .TRUE.
      planck      = .false.
      prndis(1)   = .false.
      prndis(2)   = .false.
      prndis(3)   = .false.
      prndis(4)   = .false.
      prndis(5)   = .false.
      prndis(6)   = .false.
      prndis(7)   = .false.
      prntwo(1)   = .false.
      prntwo(2)   = .false.
      IF ( solver .EQ. 3) THEN
         DO ip = 1, nprndis
            prntwo(prntdis(ip)) = .true.
         ENDDO
      ELSE
         DO ip = 1, nprndis
            prndis(prntdis(ip)) = .true.
         ENDDO
      ENDIF
      quiet       = .false.
      radius      = 6370.0
      spher       = .TRUE.
      temis       = 0.0
      ttemp       = 0.0
      usrang      = .FALSE.
      IF (numu .GT. 0 ) THEN
         onlyfl = .FALSE.
         usrang = .TRUE.
      ENDIF
      wvnmhi      = 0.0
      wvnmlo      = 0.0
cakyp
*      write(*,*)"Debug trans:", numu, usrang, (umu(iu), iu=1,numu)
*      write(*,*)"trans:", path(1:npath)
*      write(*,*)"trans:", nlyr, nlambda_t, npath
*      DO iv = 1, nlambda_t
*         WRITE(*,*)"TRANS", iv, lambda_t(iv)
*      ENDDO      
cakyp
*      nlyr = 1
*      temper(0) = 220.
*      temper(1) = 220.
*      nlambda_t = 3
*      lambda_t(1) = 300.
*      lambda_t(2) = 320.
*      lambda_t(3) = 340.
*
*
* Ozone absorption
*
      type        = o3_type
      CALL crs_o3( abs_crs, error, first_crs, lambda_t, 
     $     nlambda_t, nlyr, path(1:npath), temper, type)  
cakyp
*      lu = 1
*      DO iv = 1, nlambda_t
*         WRITE(12,*) lambda_t(iv), abs_crs(lu, io3, iv)
*      ENDDO
*      STOP
cakyp
*
* Rayleigh scattering
*
      CALL rayleigh( ray_crs, error, lambda_t, nlambda_t )
*      write(*,*)"Debug trans after rayleigh:"
cakyp
*      lu = 1
*      DO iv = 1, nlambda_t
*         WRITE(*,*)"TRANS", lambda_t(iv), abs_crs(lu, io3, iv),
*     $        ray_crs(iv)
*      ENDDO
*      STOP
cakyp
*
* Calculate transmittance for all wavelengths nlambda_t
*
      IF ( nzout .EQ. 1 .AND. zout(nzout) .EQ. 0.0 ) THEN
*         write(*,*)"Debug trans:",nzout
*         write(*,*)"Debug trans:",nlyr
*         write(*,*)"Debug trans:",zd(nlyr-1)
         nzout       = 1
         zout(nzout) = zd(nlyr)
*         write(*,*)"Debug trans:",zout(nzout)
      ENDIF
      ntau        = nzout
      usrtau      = .TRUE.
*
      DO iv = 1, nlambda_t
cakyp
*         write(*,*)"Debug UMU0",iv, lambda_t(iv), sza(iv)
cakyp
         CALL opt_prop(abs_crs(0,1,iv), aer_dtau(1,iv), aer_gg(1,iv),
     $        aer_ssa(1,iv), dens, dtauc, ice_dtau(1,iv), ice_gg(1,iv),
     $        ice_ssa(1,iv), intgr, nlyr, nstr, pmom, ray_crs(iv),
     $        ssalb, wc_dtau(1,iv), wc_gg(1,iv), wc_ssa(1,iv), zd)
*
         umu0        = cos(sza(iv)*pi/180.)
cakyp
*            write(*,*)"UMU0",iv, lambda_t(iv), sza(iv), umu0
*            prndis(1) = .TRUE.
*            prndis(2) = .TRUE.
*            prndis(3) = .TRUE.
*            prndis(7) = .TRUE.
cakyp
         CALL setout( dtauc, nlyr, ntau, utau, zd, zout)
*
cakyp
*         write(*,*)"calling disort", iv, albedo(iv)
cakyp
*     
         IF ( solver .EQ. 1 ) THEN
         CALL  disort( nlyr, dtauc, ssalb, pmom, ncoef, temper, 
     $     wvnmlo, wvnmhi, usrtau, ntau, utau, nstr, 
     $     usrang, numu, umu, nphi, phi, ibcnd, fbeam, 
     $     umu0, phi0(iv), fisot, lamber, albedo(iv), hl, btemp, 
     $     ttemp, temis, deltam, imthd, planck, onlyfl, 
     $     accur, quiet, ierror_d, prndis, header, nlyr, 
     $     nzout, maxumu, maxcmu, maxcoe, maxphi, 
     $     rfldir(1,iv), rfldn(1,iv), flup(1,iv), dfdt(1,iv), 
     $     uavg(1,iv), uu(1,1,1,iv), u0u(1,1,iv), albmed, trnmed, 
     $     uavgdn(1,iv), uavgso(1,iv), uavgup(1,iv) )
*
*         write(*,*)"trans",iv,(u0u(iu,1,iv),iu=1,numu)
*
*         write(*,*)"trans",iv,rfldir(1,iv),rfldn(1,iv), uavg(1,iv)
*         write(*,*)"trans",iv,rfldir(2,iv),rfldn(2,iv), uavgup(2,iv)
         ELSE IF ( solver .EQ. 2 ) THEN
            CALL  sdisort( nlyr, dtauc, ssalb, pmom, temper, wvnmlo,
     $           wvnmhi, usrtau, ntau, utau, nstr, usrang, numu, 
     $           umu, nphi, phi, fbeam, beta, nil, umu0, phi0(iv), 
     $           newgeo, zd, spher, radius, fisot, albedo(iv), btemp, 
     $           ttemp, temis, deltam, planck, onlyfl, accur, quiet, 
     $           ierror_s, prndis, header, nlyr, nzout, maxumu, 
     $           maxcmu, maxphi, rfldir(1,iv), rfldn(1,iv), flup(1,iv),
     $           dfdt(1,iv), uavg(1,iv), uu(1,1,1,iv), u0u(1,1,iv), 
     $           uavgdn(1,iv), uavgso(1,iv), uavgup(1,iv) )
         ELSE IF ( solver .EQ. 3 ) THEN
            DO  lc = 1, nlyr
               gg(lc) = pmom(1,lc)
            ENDDO
            CALL  twostr( albedo(iv), btemp, deltam, dtauc, fbeam, 
     $           fisot,
     $           gg, header, ierror_t, nlyr, nzout, newgeo, nlyr,
     $           planck, ntau, prntwo, quiet, radius, spher, ssalb, 
     $           temis, temper, ttemp, umu0,  usrtau, utau, wvnmlo,
     $           wvnmhi, zd, dfdt(1,iv), flup(1,iv), rfldir(1,iv), 
     $           rfldn(1,iv), uavg(1,iv) )
         ENDIF
*     
      ENDDO
*      Do iv=1, nlambda_t
*      do lu=1, nzout
*         write(*,*)"trans b",iv,rfldir(1,iv),rfldn(1,iv), uavg(1,iv)
*         rfldir(lu,iv)= rfldir(lu,iv)
*         rfldir(lu,iv)= lu + iv*100
*         rfldn(lu,iv)= lu + iv*100
*         flup(lu,iv)= lu + iv*100
*         uavg(lu,iv)= lu + iv*100
*         uavgso(lu,iv)= lu + iv*100
*         uavgdn(lu,iv)= lu + iv*100
*         uavgup(lu,iv)= lu + iv*100
*      enddo
*      enddo
*
      RETURN
      END
